/**************************************************************************
	Per USB knnen 6 Servos gesteuert werden z.B. mit:
	 usb_control_msg(USB_DEVICE, 4, ServoNummer, Winkelwert, buffer, 0, 1000);

	im AVR mssen CKSEL0, SUT0, BOOTSZ0, BOOTSZ1 und SPIEN gesetzt werden

	Die Steuerung der Servos wurde zum Teil von Stochri aus dem Thread:
	http://www.roboternetz.de/phpBB2/zeigebeitrag.php?t=14220&highlight=servo
	bernommen

	Ein Testprogramm in Delphi fand ich bei W.Schulte http://www.delta7.de/impressum.php

	Einen funktionierenden USB-Treiber fand ich auf http://www.modding-faq.de/index.php?artid=615

	Und robocat hat auf http://www.roboternetz.de
	einen schnen Beitrag geschrieben der beim Einstieg in das Thema hilft
	http://www.roboternetz.de/phpBB2/zeigebeitrag.php?t=42073
	
	Vielen Dank an alle!


; ================================================
;           Hardware connections
; ================================================
;                 
; Hardware
;              ___________
;       __    /           |      
;+5V O-|__|--|RESET    PC5|--O Servo 6
;            |            |      
;   USB   O--|PD0      PC4|--O Servo 5
;            |            |      
;         O--|PD1      PC3|--O Servo 4
;            |            |     
;   USB   O--|PD2      PC2|--O Servo 3
;            |            |
;         O--|PD3     ADC1|--O Servo 2 
;            |    A T     |
;         O--|PD4     ADC0|--O Servo 1
;            |   mega     |
;     +5V O--|VCC      GND|--O GND
;            |     8      |    
;     GND O--|GND     AREF|--O
;            |            |      
;   XTAL1 O--|XTAL1   AVCC|--O
;            |            |
;   XTAL2 O--|XTAL2 PB5SCK|--O 
;            |            |
;   frei  O--|PD5  PB4MISO|--O 
;            |            |
;   frei  O--|PD6  PB3MOSI|--O 
;            |            |
;   frei  O--|PD7  PB2OC1B|--O 
;            |            |
;         O--|PB0  PB1OC1A|--O 
;            |____________|
;
;
*/ 

/* Deklarationen */
/**************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "lcd.h"
#include "BinaryNr.h"
#include "usbdrv.h"
//#include "anzeige.h"
#include "ausgelagert.h"
#include <stdlib.h>
#include <inttypes.h>
#include <util/delay.h>

typedef unsigned char byte;
typedef struct {
	uchar increase:1;
	uchar shift:1;
	uchar C_on:1;
	uchar B_on:1;
	uchar D_shift:1;
	uchar R_shift:1;
	uchar lines:1;
	uchar font:1;
} LCD_Options_t;

#define SYSCLK 12000000 // timer clock 12Mhz
#define MAXPULSFREQ 500 // 2ms => 500HZ
#define USB_CFG_CLOCK_KHZ 12000  //SYSCLK/1000

#define TIMER_MAXPULS SYSCLK/MAXPULSFREQ // Timer1 value for a 2ms Puls

#define MINPULS TIMER_MAXPULS/4  // min pulslength = 0.5ms
#define MAXPULS TIMER_MAXPULS // max pulslength=2ms

// Definition der Pins fr die Servos

#define LOW_SERVO0 PORTC&=~(1<<0)
#define HIGH_SERVO0 PORTC|=(1<<0)

#define LOW_SERVO1 PORTC&=~(1<<1)
#define HIGH_SERVO1 PORTC|=(1<<1)

#define LOW_SERVO2 PORTC&=~(1<<2)
#define HIGH_SERVO2 PORTC|=(1<<2)

#define LOW_SERVO3 PORTC&=~(1<<3)
#define HIGH_SERVO3 PORTC|=(1<<3)

#define LOW_SERVO4 PORTC&=~(1<<4)
#define HIGH_SERVO4 PORTC|=(1<<4)

#define LOW_SERVO5 PORTC&=~(1<<5)
#define HIGH_SERVO5 PORTC|=(1<<5)

#define msgOK 0
#define msgErr 1

// Auf Kanal 4 kommen die Winkeldaten an
// Auf Kanal 3 der Befehl LCD-Clear
// bentigt von usbFunctionSetup

#define cmd_Data 4
#define cmdLCD_CLS 3

static uchar replybuffer[2];
static int angle;				// Winkel
static int channel;				// Servokanal
int volatile angnr[7];       	// Array fr Anzeige der 6 WinkelWerte

uint16_t Pulslength[20];        // Array fr delays

byte servos[10];


/* INIT */
/************************************************************************

   void init(void)
   initialize the prozessor registers

***************************************************************************/

void init(void)
{

    /* initialize ports */

    DDRC  = B00111111;                // PORTC 0 bis 5 als Ausgang
    DDRD  = B01111000;
    DDRB  = B00000000;
	PORTB = 0x00;
    PORTC = 0x00;
    PORTD = 0x00;

    // init timer1
    TCNT1 = 0-16000;
    TCCR1A=0;
    TCCR1B=0x01;
    TIMSK |= _BV(TOIE2) | _BV(TOIE1);

}


/* Routinen */
/************************************************************************

   SIGNAL(SIG_OVERFLOW1)
   timer1 interrupt, Pulserzeugung 

***************************************************************************/
SIGNAL(SIG_OVERFLOW1)
{
   static byte servoindex_half=0;

   switch (servoindex_half)
   {
      case 0: HIGH_SERVO0; break;
      case 1: LOW_SERVO0; break;
      case 2: HIGH_SERVO1; break;
      case 3: LOW_SERVO1; break;
      case 4: HIGH_SERVO2; break;
      case 5: LOW_SERVO2; break;
      case 6: HIGH_SERVO3; break;
      case 7: LOW_SERVO3; break;
      case 8: HIGH_SERVO4; break;
      case 9: LOW_SERVO4; break;
      case 10: HIGH_SERVO5; break;
      case 11: LOW_SERVO5; break;
   }

   
   TCNT1 =Pulslength[servoindex_half]; 			// Zeit fr den nchsten Interrupt   

    servoindex_half++; 							// erhhe Zeitwert-Index
   if(servoindex_half==20)servoindex_half=0;   	// reset Zeitwert-Index
}


/************************************************************************

   void setservo(byte index, byte value)

   Setze die Servo-Position
   Wertebereich: 0..255

***************************************************************************/

void setservo(byte index, byte value)
{
   uint16_t wert;

   wert=MINPULS+(MAXPULS-MINPULS)/256*value;
   
   // Berechne High-Zeit
   Pulslength[index<<1]=0-wert;
   
   // Die Summe der LOW- und Highzeit fr einen Servo ist 2ms
   Pulslength[(index<<1)+1]=0-(TIMER_MAXPULS-wert);
   
   // 10 Servos ergeben 10*2ms=20ms Gesamtzykluszeit
}


/************************************************************************

	usbFunctionWrite(uchar *data, uchar len)
	Daten auf USB schreiben (wird von usbdrv.c benutzt

***************************************************************************/

uchar usbFunctionWrite(uchar *data, uchar len){
	uchar i;
	
	for (i=0;i<len;i++){
		;
	}
	return 0;
}


/************************************************************************

	usbFunctionSetup(uchar data[8])
	Diese Funktion wird vom USB-Treiber aufgerufen wenn Daten empfangen wurden

***************************************************************************/

uchar	usbFunctionSetup(uchar data[8]){
	uchar len;

	replybuffer[0] = msgOK;
	switch (data[1]) {

        case cmdLCD_CLS:					// LCD lschen
			
			len = 0xFF;

			break;

        case cmd_Data:						// Wenn USB Daten empfngt 
            len = 1;						
			  channel = data[2];			// wird der Kanal ausgelesen
			  angle = data[4];				// und der Winkelwert
			  angnr[channel]=angle;			// abgespeichert in Array fr Anzeige
			  servos[channel-1] = angle;	// und in Array fr Servo
//			  Anzeige();					// Anzeige aktualisieren
            break;
		
		default:
			replybuffer[0] = msgErr;
			len = 1;
			break;
	}
	usbMsgPtr = replybuffer;
	return len;
}


/************************************************************************

	Hauptprogramm

***************************************************************************/

int main()
{
  byte n;

  init();                           // Ports und Timer initialisieren

  //lcd_init(LCD_DISP_ON);            // initialisiere display, cursor off 

  usbInit();                        // initialisiere usb-port

  sei();							// Interrupts freigeben  

  _delay_ms(15);					// kurz warten

  angnr[1] = 111;					// Winkel Vorbesetzung fr Anzeige
  angnr[2] = 112;
  angnr[3] = 113;
  angnr[4] = 114;
  angnr[5] = 115;
  angnr[6] = 116;

  for(n=0;n<6;n++) 
  {  
       servos[n]=angnr[n+1]; 		// Winkel Vorbesetzung fr Servos
  }
 
//  Anzeige();                        // zeigt die 6 Winkel auf LCD

  for(;;)
  {
  	  usbPoll();					// USB-Port abfragen
      for(n=0;n<6;n++) 
	  {
	  	setservo(n,servos[n]);		// und Winkelwerte an Servo bergeben
 	  } 	
  }

}





